<?php
/*
Plugin Name: EduHack
Plugin URI: http://araujo.cc/portfolio/eduhack/
Description: Hacking your education! Plugin for create courses and grids.
Author: Arthur Araújo
Author URI: http://araujo.cc
Version: 1.1
*/

define( 'EDUHACK_POSTTYPE', 'eduhack' );

add_action( 'admin_menu', 'eduhack_register_menu_page' );
add_action( 'init', 'eduhack_actions' );

add_filter( 'the_content', 'eduhack_course_the_content' );

add_filter( 'wp_head', 'eduhack_show_grid' );

add_action( 'admin_enqueue_scripts', 'eduhack_add_gridnodes_js' );

# Refresh rules
register_activation_hook( __FILE__, 'eduhack_activate' );
register_deactivation_hook( __FILE__, 'eduhack_deactivate' );


/*** ACTIONS ***/

function eduhack_actions() {
	
	eduhack_register_post_type();
	
	if( !isset($_GET['page']) || !is_admin() )
		return;
	
	$page=@$_GET['page'];
	
	# EXPORT
	if( $export=@$_GET['export'] ) {
		if( is_numeric($export) && $export>0 ) {
			require_once dirname(__FILE__).'/tools.php';
			eduhack_export($export);
		}
	}
	
	# TRASH COURSE
	if( isset($_GET['token']) && is_numeric($_GET['trash']) && ($trash=@$_GET['trash']) && wp_verify_nonce($_GET['token'], 'eduhack_course_trash_'.$_GET['trash']) ) {
		if( is_numeric($trash) && $trash>0 ) {
			global $wpdb;
			$wpdb->query( $wpdb->prepare("
				UPDATE $wpdb->posts
				SET post_status='trash'
				WHERE ID=%d AND post_type='%s' AND post_parent=0
			", $trash, EDUHACK_POSTTYPE ));
		}
	}
	
	# TRASH MODULE
	if( isset($_GET['token']) && is_numeric($_GET['trash']) && wp_verify_nonce($_GET['token'], 'eduhack_module_trash_'.$_GET['trash']) ) {
		
		wp_trash_post( $_GET['trash'] );
		
		global $eduhack_message;
		$eduhack_message = __('Module moved to the Trash.', 'eduhack');
	}
	
	
	if( ( $page=='eduhack-view-course' || $page=='eduhack-add-new-module' ) && !isset($_GET['course']) )
		wp_redirect( 'admin.php?page=eduhack&message=1') && die();
	
	if( !isset($_POST['nonce']) )
		return;
	
	if( !isset($_POST['module_id']) )
		$_POST['module_id'] = '';
	#echo wp_verify_nonce( $_POST['nonce'], '_eduhack_course_'.$_POST['course_id'] );
	// add/edit course
	if( isset($_POST['course_id']) && $page=='eduhack-add-new-course' && wp_verify_nonce( $_POST['nonce'], '_eduhack_course_'.$_POST['course_id'] ) )
		eduhack_update_course( $_POST['course_name'], $_POST['course_description'], $_POST['course_id'] );
	
	// add/edit module
	elseif( isset($_POST['module_id']) && $page=='eduhack-add-new-module' && wp_verify_nonce( $_POST['nonce'], '_eduhack_module_'.$_POST['module_id'] ) )
		eduhack_module_changes( $_POST['module_id'], $_POST );
	
	// add/edit level
	elseif( isset($_POST['level']) && $page=='eduhack-add-level' && wp_verify_nonce( $_POST['nonce'], '_eduhack_add_level' ) )
		eduhack_add_level( $_POST['course_id'], $_POST['name'], $_POST['level'] );

}

/*** FUNCTIONS ***/

function eduhack_show_grid() {
	// melhorar
	if( get_post_type()!=EDUHACK_POSTTYPE || !isset($_GET['eduhack-grid']) )
		return;
		
	$script_url = plugins_url().'/eduhack/gridnodes.js';
	
	echo '<style type="text/css"> @import url("'.plugins_url().'/eduhack/grid.css"); #widgets{display:none;} #content{width:100%}</style>';
	echo "<script src='$script_url'></script>";
}

function eduhack_add_gridnodes_js() {
	
	if( @$_GET['page']=='eduhack-view-course' || isset($_GET['eduhack-grid']) ) {
		
		$script_url = plugins_url().'/eduhack/gridnodes.js';
		
		# doest work?? WHY????????
		//wp_enqueue_script('eduhack-gridnodes', $script_url, array('jQuery'));
		//add_action( 'wp_enqueue_scripts', create_function('', 'eduhack_add_gridnodes_js();') );
	
		echo "<script src='$script_url'></script>";
	}
}

function eduhack_showgrid( $course_id ) {
	
	$modules = eduhack_get_modules( $course_id );
	$levels = eduhack_get_levels( $course_id );
	?>
	<table id="eduhack-grid">
		<tr>
			<?php
			$level=0;
			$js = '';
			$modules_sort = array();
			
			foreach( $modules as $module ) {
				eduhack_setup_data($module);
				
				#print_r(eduhack_get('require'));
				
				//echo $level;
				if( $level<1 && (eduhack_get('level')=='0' || eduhack_get('level')=='1') )
					$level = 1;
				elseif( eduhack_get('level') > $level )
					$level = eduhack_get('level');
				
				$module->bgcolor = eduhack_get_module_color( $module->ID, eduhack_get('require'), eduhack_get('color') );
				$module->level = $level;
				
				$modules_sort[$level]["$module->bgcolor-$module->ID"] = $module;
			}
			
			//print_r($modules_sort);
			foreach( $modules_sort as $level=>$modules ) {
				
				$level_link = is_admin()? 'admin.php?page=eduhack-add-level&course='.$course_id.'&level='.$level.'&level_name='.$levels[$level] : '#';
				
				echo '<td valign="top" id="module_level_'.$level.'" style="padding-left:0;"><div style="text-align:center">'.__('level', 'eduhack').' '.$level.' <br /><strong><a href="'.$level_link.'">'.$levels[$level].'</a></strong></div>';
				
				ksort($modules);
				
				foreach( $modules as $module ) {
					eduhack_setup_data($module);
					
					$module_link = is_admin()? 'admin.php?page=eduhack-add-new-module&course='.$course_id.'&module='.$module->ID : get_permalink($module->ID);
					$bgcolor = $module->bgcolor;
					
					if( eduhack_get('require') )
						$js .= "connect_modules( {$module->ID}, '".eduhack_get('require')."', '{$bgcolor}', '{$level}');";
					else
						$js .= "parent_color[{$module->ID}] = '{$bgcolor}';";
					?>
					<div id="module_<?php echo $module->ID ?>" class="module" style="background-color:<?php echo $bgcolor ?>;">
						<a href="<?php echo $module_link ?>"><?php echo eduhack_get('title') ?></a>
					</div>
			<?php }
				
				echo '</td>';
				
			}
			?>
			</td>
		</tr>
	</table>

	<script type="text/javascript">
		setTimeout("<?php echo $js;?>", 1000);
	</script>
	
<?php }

function eduhack_get_level_admin_link( $course_id, $level, $levelname ) {
	return 'admin.php?page=eduhack-add-level&course='.$course_id.'&level='.$level.'&level_name='.$levelname;
}

function eduhack_showlist( $course_id, $echo=1 ) {
	
	if( isset($_GET['eduhack-grid']) )
		return eduhack_showgrid( $course_id );
	
	$modules = eduhack_get_modules( $course_id );
	$levels = eduhack_get_levels( $course_id );

	$level=0;
	$modules_sort = array();
	
	foreach( $modules as $module ) {
		eduhack_setup_data($module);

		//echo $level;
		if( $level<1 && (eduhack_get('level')=='0' || eduhack_get('level')=='1') )
			$level = 1;
		elseif( eduhack_get('level') > $level )
			$level = eduhack_get('level');
		
		#$module->bgcolor = eduhack_get_module_color( $module->ID, eduhack_get('require'), eduhack_get('color') );
		
		$module->level = $level;
		
		$modules_sort[$level][$module->ID] = $module;
		
	}
			
	echo '<table class="eduhack-list">';
	foreach( $modules_sort as $level=>$modules ) {

		# if is admin can edit levels
		if( is_admin() ) {
			$level_link = eduhack_get_level_admin_link( $course_id, $level, $levels[$level] );
			$level_link = "<a href='$level_link' style='color:#ddd'>".__('Level', 'eduhack')." $level</a>";
		} else
			$level_link = __('level', 'eduhack')." $level";
			
		echo '<tr><th valign="top" id="module_level_'.$level.'" '.(is_admin()?'colspan="3"':'').'>
				<div style="text-align:center">'.$level_link.' <br /><strong>'.$levels[$level].'</strong></div></th></tr>';
		
		#ksort($modules);
		$i=0;
		foreach( $modules as $key=>$module ) {
			eduhack_setup_data($module);
			
			$i++;
			
			#$bgcolor = $module->bgcolor;
			?>
			<tr class="<?php echo ( $i & 1 ) ? 'normalrow' : 'altrow'; ?>">
				<td>
					<div id="module_<?php echo $module->ID ?>" class="module">
						<a href="<?php echo get_permalink($module->ID) ?>" <?php if( is_admin() ) echo 'target="_blank"'; ?>>
							<?php echo eduhack_get('title') ?></a>
					</div>
				</td>
				<?php if( is_admin() ) {
					$edit_link = 'admin.php?page=eduhack-add-new-module&course='.$course_id.'&module='.$module->ID;
					$trash_link = 'admin.php?page=eduhack-view-course&course='.$course_id.'&trash='.$module->ID.'&token='.wp_create_nonce('eduhack_module_trash_'.$module->ID); ?>
					<td align="right">
						<a href="<?php echo $edit_link; ?>"><?php _e('edit', 'eduhack') ?></a> |
						<a href="<?php echo $trash_link; ?>" style="color:red" onclick="return confirm('<?php _e('Are you sure?', 'eduhack') ?>')"><?php _e('delete', 'eduhack') ?></a>
					</td>
				<?php } ?>
			</tr>
	<?php }
		
		echo '<tr><td '.(is_admin()?'colspan="3"':'').'>&nbsp;</td></tr>';
	}
	echo '</table>';
}

function eduhack_course_the_content( $content ) {
	
	global $post;
	
	// melhorar
	if( get_post_type()!=EDUHACK_POSTTYPE )
		return $content;
		
	if( is_numeric($post->post_parent) && $post->post_parent>0 ) {
		
		$header = '<div class="course_header"><strong>'.__('Course', 'eduhack').': </strong>'.get_post($post->post_parent)->post_title.'</div>';
		
		$content = $header . $content;
		
		return $content;
	}
	
	echo $content;
	
	eduhack_showlist( get_the_ID() );
	
	#wp_reset_query();
}

function eduhack_register_post_type() {
	register_post_type( EDUHACK_POSTTYPE, array(
		'labels' => array(
			'name' => 'EduHack',
		),
		'public' => true,
		'publicly_queryable' => true,
		'show_ui' => true, 
		'show_in_menu' => false, 
		'query_var' => true,
		'capability_type' => 'post',
		'has_archive' => true, 
		'hierarchical' => true,
		'menu_position' => null,
		'supports' => array( 'title', 'editor', 'author', 'thumbnail', 'excerpt', 'comments' ),
		'rewrite' => array( 'slug' => EDUHACK_POSTTYPE ),
	) );
}

function eduhack_add_level( $course_id=0, $name='', $level=0 ) {
		
		if( empty($name) )
			return false;
		
		$levels = get_option( "eduhack-$course_id-levels", array() );
		$levels[$level]=$name;
		
		update_option( "eduhack-$course_id-levels", $levels );
		
		wp_redirect( 'admin.php?page=eduhack-view-course&course='.$course_id );
}

function eduhack_register_menu_page() {
	
	add_menu_page( __('EduHack - Hacking your education', 'eduhack'), 'EduHack', 'manage_options', 'eduhack', 'eduhack_admin_page_include', null, 25 );

	if( @$_GET['page']=='eduhack-add-new-module' )
		add_submenu_page( 'eduhack', 'EduHack - Hacking your education', __('— Add New Module', 'eduhack'), 'manage_options', 'eduhack-add-new-module', 'eduhack_submenu_page2_callback' ); 
	
	if( @$_GET['page']=='eduhack-view-course' )
		add_submenu_page( 'eduhack', 'EduHack - Hacking your education', __('— View course', 'eduhack'), 'manage_options', 'eduhack-view-course', 'eduhack_submenu_page3_callback' ); 
	
	add_submenu_page( 'eduhack', 'EduHack - Hacking your education', __('Add New Course', 'eduhack'), 'manage_options', 'eduhack-add-new-course', 'eduhack_submenu_page_callback' ); 
	
	add_submenu_page( 'eduhack', 'EduHack - Hacking your education', __('Levels', 'eduhack'), 'manage_options', 'eduhack-add-level', 'eduhack_submenu_page4_callback' ); 
	
	if( is_admin() && @$_GET['page']=='eduhack-add-new-module' ) {
		wp_enqueue_script( 'wp-color-picker' );
		wp_enqueue_style( 'wp-color-picker' );
	}

}

function eduhack_admin_page_include() {
	include dirname(__FILE__).'/admin.php';
}

function eduhack_submenu_page_callback() { include dirname(__FILE__).'/admin-course.php'; }
function eduhack_submenu_page2_callback() { include dirname(__FILE__).'/admin-module.php'; }
function eduhack_submenu_page3_callback() { include dirname(__FILE__).'/admin-view-course.php'; }
function eduhack_submenu_page4_callback() { include dirname(__FILE__).'/admin-level.php'; }

function eduhack_get_courses() {
	return query_posts( "post_parent=0&post_type=".EDUHACK_POSTTYPE );
}

function eduhack_get_modules( $course_id=0, $exception='' ) {
	
	global $wpdb;
	
	$course_id = (int)$course_id;
	
	if( !empty($exception) )
		$exception = sprintf("AND ID!=%d",$exception);
	
	#return query_posts( "orderby=menu_order&order=ASC&post_parent=$course_id&posts_per_page=10000&post_type=".EDUHACK_POSTTYPE );

	return $wpdb->get_results( $wpdb->prepare( "
		SELECT ID, post_title, post_excerpt, post_parent, post_content, post_date, post_author, menu_order, meta_value AS 'require'
		FROM $wpdb->posts
			LEFT JOIN $wpdb->postmeta ON post_id=ID AND meta_key='require'
		WHERE post_parent='%d' AND post_type='%s' AND post_status='publish' {$exception}
		ORDER BY menu_order ASC
		
	", $course_id, EDUHACK_POSTTYPE ) );
	
}

function eduhack_module_changes( $module_id=null, $data=null ) {
	
	global $user_ID;
	
	$course_id = (int) $data['course_id'];
	
	if( !is_numeric($course_id) || $course_id==0 )
		return;
	
	// setup data
	$postdata = array(
	  'ID'			  => $module_id,
	  'post_title'    => $data['name'],
	  'post_excerpt'  => $data['color'],
	  'post_content'  => $data['description'],
	  'post_type' 	  => EDUHACK_POSTTYPE,
	  'post_status'   => 'publish',
	  'post_author'   => $user_ID,
	  'post_parent'   => $course_id,
	  'menu_order'   => (int)$data['level'],
	);
	
	// update or add new module
	if( is_numeric($module_id) )
		wp_update_post( $postdata );
	else
		$module_id = wp_insert_post( $postdata );
	
	#print_r( implode( ',', $data['require'] ).',' ); exit;
	
	update_post_meta( $module_id, 'require', implode( ',', (array)$data['require'] ) );
	
	// redirect to view course page
	wp_redirect( 'admin.php?page=eduhack-view-course&course='.$course_id );
}

function eduhack_add_new_course( $course_name='', $course_description='' ) {
	
	if( empty($course_name) )
		return false;
	
	eduhack_update_course( $course_name='', $course_description='' );
}

function eduhack_update_course( $course_name='', $course_description='', $course_id=null ) {
	
	if( empty($course_name) )
		return false;
	
	global $user_ID;
	
	$data = array(
	  'ID'    => $course_id,
	  'post_title'    => $course_name,
	  'post_excerpt'  => $course_description,
	  'post_content'  => $course_description,
	  'post_type'  	  => EDUHACK_POSTTYPE,
	  'post_status'   => 'publish',
	  'post_author'   => $user_ID,
	);
	
	wp_insert_post( $data );
	
	wp_redirect( 'admin.php?page=eduhack' );
}

function eduhack_get_levels( $course_id=0 ) {
	return get_option( "eduhack-$course_id-levels", array() );
}

/* FORMS */

function eduhack_setup_data( $field_id ) {
	global $eduhack_data, $wpdb;
	
	if( is_null($field_id) )
		return;
	
	if( is_object($field_id) )
		$eduhack_data=$field_id;
	else
		$eduhack_data=$wpdb->get_row( $wpdb->prepare("SELECT * FROM $wpdb->posts WHERE ID='%d'", $field_id) );
}
function eduhack_get( $fieldname, $type='course', $default='' ) {
	global $eduhack_data;
	
	// Eduhack language to WP language estructure
	if( $fieldname=='title' )
		$fieldname = 'post_title';
	elseif( $fieldname=='description' )
		$fieldname = 'post_content';
	elseif( $fieldname=='level' )
		$fieldname = 'menu_order';
	elseif( $fieldname=='course_id' )
		$fieldname = 'post_parent';
	elseif( $fieldname=='module_id' )
		$fieldname = 'ID';
	elseif( $fieldname=='color' )
		$fieldname = 'post_excerpt';
	
	if( is_null($eduhack_data) )
		return $default;
	elseif( isset($eduhack_data->$fieldname) )
		return $eduhack_data->$fieldname;
}

function eduhack_selected( $fieldname, $value, $echo=1 ) {
	global $eduhack_data;
	
	if( isset($_GET['course']) && $_GET['course']==$value )
		$code = 'selected="selected"';
	elseif( !is_null($eduhack_data) && eduhack_get( $fieldname )==$value )
		$code = 'selected="selected"';
	else 
		return '';
		
	if( $echo )
		echo $code;
	else
		return $code;
}

function eduhack_checked( $value, $echo=1 ) {
	global $eduhack_data;
	
	if( is_null($eduhack_data) )
		return '';
	
	// setup require modules var
	if( !isset($eduhack_data->tags) ) {
		
		$eduhack_data->require = get_post_meta( $eduhack_data->ID, 'require', 1 ).',';
		//echo $eduhack_data->ID.'(-';
	}
	
	if( strpos($eduhack_data->require, $value.',')!==FALSE )
		$code = 'checked="checked"';
	else 
		return '';
		
	if( $echo )
		echo $code;
	else
		return $code;
}

function eduhack_get_module_color( $module_id, $require, $color='' ) {
	
	global $eduhack_colors;
	
	if( $color!='' ) {
		
		$eduhack_colors[$module_id] = array($color, $require);
		
		return $color;
		
	} elseif( $require!='' ) {
		$modules = explode( ',', $require );
		
		foreach( $modules as $parent_module_id ):
			if( isset($eduhack_colors[$parent_module_id]) ) {
				if( $eduhack_colors[$parent_module_id][0]!='' ) {
					
					// get the color
					$color = $eduhack_colors[$parent_module_id][0];
					
					// save the parent color
					$eduhack_colors[$module_id] = array($color);
					
					// return the color
					return $color;
				}
			}
		endforeach;
	}	
		
	// return default module color
	return '#999';
}

function count_modules( $course_id ) {
	global $wpdb;
	
	if( !$course_id )
		return 0;
	
	$count=$wpdb->get_var( $wpdb->prepare("SELECT COUNT(ID) FROM $wpdb->posts WHERE post_parent='%d' AND post_type='%s'", $course_id, EDUHACK_POSTTYPE) );
	
	return $count;
}

function eduhack_activate() {
	eduhack_register_post_type();
	flush_rewrite_rules();
}

function eduhack_deactivate() {
	flush_rewrite_rules();
}

// translate to PT

if( is_admin() && defined('WPLANG') && ( WPLANG=='pt_BR' || WPLANG=='pt_PT' ) ) {
	add_filter( 'gettext', 'eduhack_pt_translate', 10, 3 );
	
	global $eduhack_pt;
	
	$eduhack_pt = array(
		'EduHack - Hacking your education' => 'EduHack - Hackeiando a educação!',
		'Add New Course' => 'Adicionar Curso',
		'Add New' => 'Adicionar',
		'Add New Module' => 'Adicionar Módulo',
		'Add Module' => 'Adicionar Módulo',
		'Add Level' => 'Adicionar Nível',
		'View Grid' => 'Ver Grade',
		'View as List' => 'Ver em Lista',
		'— Add New Module' => '— Adicionar módulo',
		'— View course' => '— Ver curso',
		'Level' => 'Nível',
		'Levels' => 'Níveis',
		'Module moved to the Trash.' => 'Módulo movido para a lixeira.',
		'Course' => 'Curso',
		'Courses' => 'Cursos',
		'Are you sure?' => 'Tem certeza disto?',
		'edit' => 'editar',
		'Edit' => 'Editar',
		'Edit Course' => 'Editar Curso',
		'delete' => 'excluir',
		'level' => 'nível',
		'Write something about this course...' => 'Escreva algo sobre o curso...',
		'Description' => 'Descrição',
		'Enter course name here' => 'Digite o nome do curso',
		'Save Course' => 'Salvar Curso',
		'Save Module' => 'Salvar Módulo',
		'Save Level' => 'Salvar Nível',
		'Select a course' => 'Selecione um curso',
		'Select a level' => 'Selecione um nível',
		'Select a course below...' => 'Selecione um dos cursos abaixo...',
		'Enter level name here' => 'Digite o nome do curso',
		'Enter module name here' => 'Digite o nome do módulo',
		'Module color' => 'Cor do módulo',
		'Modules' => 'Módulos',
		'Requirements' => 'Requisitos',
		'Requirements' => 'Requisitos',
		'View' => 'Ver',
		'View page' => 'Ver página',
		'View Course' => 'Ver curso',
	);
}

function eduhack_pt_translate( $translated_text, $text, $domain ) {
	
	if( $domain!='eduhack' )
		return $translated_text;
	
	global $eduhack_pt;
	
	$array = $eduhack_pt;
	
    if( isset($array[$translated_text]) )
		return $array[$translated_text];
	else
		return $translated_text;
}
